@@ -1,7 +1,9 @@
 import os
 import socket
 from libensemble.resources.env_resources import EnvResources
-from libensemble.resources.resources import Resources, WorkerResources, ResourcesException
+from libensemble.resources.resources import Resources, GlobalResources, ResourcesException
+from libensemble.resources.worker_resources import ResourceManager, WorkerResources
+from libensemble.resources.mpi_resources import create_machinefile
 
 
 def setup_standalone_run():
@@ -45,7 +47,7 @@ def sname(name):
 
 # Tests ========================================================================================
 
-# Tests Resources.get_global_nodelist (This requires above tests to work)
+# Tests GlobalResources.get_global_nodelist (This requires above tests to work)
 def test_get_global_nodelist_frm_slurm():
     os.environ["LIBE_RESOURCES_TEST_NODE_LIST"] = "knl-[0020-0022,0137-0139,1234]"
     exp_out = ['knl-0020', 'knl-0021', 'knl-0022', 'knl-0137', 'knl-0138', 'knl-0139', 'knl-1234']
@@ -53,7 +55,7 @@ def test_get_global_nodelist_frm_slurm():
                                  nodelist_env_cobalt="THIS_ENV_VARIABLE_IS_DEF_NOT_SET",
                                  nodelist_env_lsf="THIS_ENV_VARIABLE_IS_DEF_NOT_SET",
                                  nodelist_env_lsf_shortform="THIS_ENV_VARIABLE_IS_DEF_NOT_SET")
-    global_nodelist = Resources.get_global_nodelist(rundir=os.getcwd(), env_resources=env_resources)
+    global_nodelist = GlobalResources.get_global_nodelist(rundir=os.getcwd(), env_resources=env_resources)
     assert global_nodelist == exp_out, \
         "global_nodelist returned does not match expected. \nRet: {}\nExp: {}".format(global_nodelist, exp_out)
 
@@ -66,7 +68,7 @@ def test_get_global_nodelist_frm_slurm_suffix():
                                  nodelist_env_cobalt="THIS_ENV_VARIABLE_IS_DEF_NOT_SET",
                                  nodelist_env_lsf="THIS_ENV_VARIABLE_IS_DEF_NOT_SET",
                                  nodelist_env_lsf_shortform="THIS_ENV_VARIABLE_IS_DEF_NOT_SET")
-    global_nodelist = Resources.get_global_nodelist(rundir=os.getcwd(), env_resources=env_resources)
+    global_nodelist = GlobalResources.get_global_nodelist(rundir=os.getcwd(), env_resources=env_resources)
     assert global_nodelist == exp_out, \
         "global_nodelist returned does not match expected. \nRet: {}\nExp: {}".format(global_nodelist, exp_out)
 
@@ -78,7 +80,7 @@ def test_get_global_nodelist_frm_slurm_single():
                                  nodelist_env_cobalt="THIS_ENV_VARIABLE_IS_DEF_NOT_SET",
                                  nodelist_env_lsf="THIS_ENV_VARIABLE_IS_DEF_NOT_SET",
                                  nodelist_env_lsf_shortform="THIS_ENV_VARIABLE_IS_DEF_NOT_SET")
-    global_nodelist = Resources.get_global_nodelist(rundir=os.getcwd(), env_resources=env_resources)
+    global_nodelist = GlobalResources.get_global_nodelist(rundir=os.getcwd(), env_resources=env_resources)
     assert global_nodelist == exp_out, \
         "global_nodelist returned does not match expected. \nRet: {}\nExp: {}".format(global_nodelist, exp_out)
 
@@ -90,7 +92,7 @@ def test_get_global_nodelist_frm_slurm_straight_list():
                                  nodelist_env_cobalt="THIS_ENV_VARIABLE_IS_DEF_NOT_SET",
                                  nodelist_env_lsf="THIS_ENV_VARIABLE_IS_DEF_NOT_SET",
                                  nodelist_env_lsf_shortform="THIS_ENV_VARIABLE_IS_DEF_NOT_SET")
-    global_nodelist = Resources.get_global_nodelist(rundir=os.getcwd(), env_resources=env_resources)
+    global_nodelist = GlobalResources.get_global_nodelist(rundir=os.getcwd(), env_resources=env_resources)
     assert global_nodelist == exp_out, \
         "global_nodelist returned does not match expected. \nRet: {}\nExp: {}".format(global_nodelist, exp_out)
 
@@ -103,7 +105,7 @@ def test_get_global_nodelist_frm_slurm_multigroup():
                                  nodelist_env_cobalt="THIS_ENV_VARIABLE_IS_DEF_NOT_SET",
                                  nodelist_env_lsf="THIS_ENV_VARIABLE_IS_DEF_NOT_SET",
                                  nodelist_env_lsf_shortform="THIS_ENV_VARIABLE_IS_DEF_NOT_SET")
-    global_nodelist = Resources.get_global_nodelist(rundir=os.getcwd(), env_resources=env_resources)
+    global_nodelist = GlobalResources.get_global_nodelist(rundir=os.getcwd(), env_resources=env_resources)
     assert global_nodelist == exp_out, \
         "global_nodelist returned does not match expected. \nRet: {}\nExp: {}".format(global_nodelist, exp_out)
 
@@ -115,7 +117,7 @@ def test_get_global_nodelist_frm_slurm_complex():
                                  nodelist_env_cobalt="THIS_ENV_VARIABLE_IS_DEF_NOT_SET",
                                  nodelist_env_lsf="THIS_ENV_VARIABLE_IS_DEF_NOT_SET",
                                  nodelist_env_lsf_shortform="THIS_ENV_VARIABLE_IS_DEF_NOT_SET")
-    global_nodelist = Resources.get_global_nodelist(rundir=os.getcwd(), env_resources=env_resources)
+    global_nodelist = GlobalResources.get_global_nodelist(rundir=os.getcwd(), env_resources=env_resources)
     assert global_nodelist == exp_out, \
         "global_nodelist returned does not match expected. \nRet: {}\nExp: {}".format(global_nodelist, exp_out)
 
@@ -127,7 +129,7 @@ def test_get_global_nodelist_frm_cobalt():
                                  nodelist_env_cobalt="LIBE_RESOURCES_TEST_NODE_LIST",
                                  nodelist_env_lsf="THIS_ENV_VARIABLE_IS_DEF_NOT_SET",
                                  nodelist_env_lsf_shortform="THIS_ENV_VARIABLE_IS_DEF_NOT_SET")
-    global_nodelist = Resources.get_global_nodelist(rundir=os.getcwd(), env_resources=env_resources)
+    global_nodelist = GlobalResources.get_global_nodelist(rundir=os.getcwd(), env_resources=env_resources)
     assert global_nodelist == exp_out, \
         "global_nodelist returned does not match expected. \nRet: {}\nExp: {}".format(global_nodelist, exp_out)
 
@@ -139,7 +141,7 @@ def test_get_global_nodelist_frm_lsf():
                                  nodelist_env_cobalt="THIS_ENV_VARIABLE_IS_DEF_NOT_SET",
                                  nodelist_env_lsf="LIBE_RESOURCES_TEST_NODE_LIST",
                                  nodelist_env_lsf_shortform="THIS_ENV_VARIABLE_IS_DEF_NOT_SET")
-    global_nodelist = Resources.get_global_nodelist(rundir=os.getcwd(), env_resources=env_resources)
+    global_nodelist = GlobalResources.get_global_nodelist(rundir=os.getcwd(), env_resources=env_resources)
     assert global_nodelist == exp_out, \
         "global_nodelist returned does not match expected. \nRet: {}\nExp: {}".format(global_nodelist, exp_out)
 
@@ -151,7 +153,7 @@ def test_get_global_nodelist_frm_lsf_shortform():
                                  nodelist_env_cobalt="THIS_ENV_VARIABLE_IS_DEF_NOT_SET",
                                  nodelist_env_lsf="THIS_ENV_VARIABLE_IS_DEF_NOT_SET",
                                  nodelist_env_lsf_shortform="LIBE_RESOURCES_TEST_NODE_LIST")
-    global_nodelist = Resources.get_global_nodelist(rundir=os.getcwd(), env_resources=env_resources)
+    global_nodelist = GlobalResources.get_global_nodelist(rundir=os.getcwd(), env_resources=env_resources)
     assert global_nodelist == exp_out, \
         "global_nodelist returned does not match expected. \nRet: {}\nExp: {}".format(global_nodelist, exp_out)
 
@@ -163,7 +165,7 @@ def test_get_global_nodelist_standalone():
                                  nodelist_env_cobalt="THIS_ENV_VARIABLE_IS_DEF_NOT_SET",
                                  nodelist_env_lsf="THIS_ENV_VARIABLE_IS_DEF_NOT_SET",
                                  nodelist_env_lsf_shortform="THIS_ENV_VARIABLE_IS_DEF_NOT_SET")
-    global_nodelist = Resources.get_global_nodelist(rundir=os.getcwd(), env_resources=env_resources)
+    global_nodelist = GlobalResources.get_global_nodelist(rundir=os.getcwd(), env_resources=env_resources)
     assert global_nodelist == [exp_node], "global_nodelist returned does not match expected"
 
 
@@ -174,7 +176,7 @@ def test_get_global_nodelist_frm_wrklst_file():
 
     open('node_list', 'w').close()
     try:
-        _ = Resources.get_global_nodelist(rundir=os.getcwd())
+        _ = GlobalResources.get_global_nodelist(rundir=os.getcwd())
     except ResourcesException as e:
         assert e.args[0] == 'Error. global_nodelist is empty'
     else:
@@ -185,7 +187,7 @@ def test_get_global_nodelist_frm_wrklst_file():
             f.write(node + '\n')
 
     # Do not specify env vars.
-    global_nodelist1 = Resources.get_global_nodelist(rundir=os.getcwd())
+    global_nodelist1 = GlobalResources.get_global_nodelist(rundir=os.getcwd())
     assert global_nodelist1 == exp_out, "global_nodelist returned does not match expected"
 
     # Specify env vars - should ignore
@@ -193,7 +195,7 @@ def test_get_global_nodelist_frm_wrklst_file():
                                  nodelist_env_cobalt="LIBE_RESOURCES_TEST_NODE_LIST",
                                  nodelist_env_lsf="THIS_ENV_VARIABLE_IS_DEF_NOT_SET",
                                  nodelist_env_lsf_shortform="THIS_ENV_VARIABLE_IS_DEF_NOT_SET")
-    global_nodelist2 = Resources.get_global_nodelist(rundir=os.getcwd(), env_resources=env_resources)
+    global_nodelist2 = GlobalResources.get_global_nodelist(rundir=os.getcwd(), env_resources=env_resources)
     assert global_nodelist2 == exp_out, "global_nodelist returned does not match expected"
     os.remove('node_list')
 
@@ -204,7 +206,7 @@ def test_remove_libE_nodes():
 
     # Add at beginning
     nodes_in = [mynode] + exp_out
-    nodes_out = Resources.remove_nodes(nodes_in, mynode)
+    nodes_out = GlobalResources.remove_nodes(nodes_in, mynode)
     assert nodes_out == exp_out, "nodelist returned does not match expected"
 
     # Add twice in middle and at end
@@ -213,52 +215,134 @@ def test_remove_libE_nodes():
         nodes_in.append(node)
         if i == 1 or i == 4 or i == 6:
             nodes_in.append(mynode)
-    nodes_out = Resources.remove_nodes(nodes_in, mynode)
+    nodes_out = GlobalResources.remove_nodes(nodes_in, mynode)
     assert nodes_out == exp_out, "nodelist returned does not match expected"
 
 
-def test_get_local_nodelist_central_mode():
+def _assert_worker_attr(wres, attr, exp):
+    ret = getattr(wres, attr)
+    assert ret == exp, "{} returned does not match expected.  \nRet: {}\nExp: {}".format(attr, ret, exp)
+
+
+# These are all 1 worker per rset.
+def _worker_asserts(wres, split_list, exp_slots, wrk, nworkers, nnodes, reps=1):
+
+    # Create dictionary of attributes and expected values
+    exp_dict = {'workerID': wrk + 1,
+                'local_nodelist': split_list[wrk],
+                'slots': exp_slots,
+                'local_node_count': nnodes,
+                'num_workers': nworkers,
+                'split_list': split_list,
+                'slot_count': 1,
+                'local_rsets_list': [reps] * nworkers,
+                'rsets_per_node': reps,
+                }
+
+    for attr, exp_val in exp_dict.items():
+        _assert_worker_attr(wres, attr, exp_val)
+
+
+# These are all >= 1 node per rset. And 1 worker per rset
+# dedicated_mode makes no difference in this test
+def test_get_local_resources_dedicated_mode():
     os.environ["LIBE_RESOURCES_TEST_NODE_LIST"] = "knl-[0020-0022,0036,0137-0139,1234]"
-    resources = Resources(nodelist_env_slurm="LIBE_RESOURCES_TEST_NODE_LIST", central_mode=True)
+    resource_info = {'nodelist_env_slurm': "LIBE_RESOURCES_TEST_NODE_LIST"}
+    libE_specs = {'resource_info': resource_info,
+                  'dedicated_mode': True}
+    gresources = GlobalResources(libE_specs)
 
-    # Now mock up some more stuff - so consistent
-
-    # Spoof current process as each worker and check nodelist.
-    num_workers = 8
+    # 8 Workers ---------------------------------------------------------------
+    nworkers = 8
     exp_out = [['knl-0020'], ['knl-0021'], ['knl-0022'], ['knl-0036'],
                ['knl-0137'], ['knl-0138'], ['knl-0139'], ['knl-1234']]
-    for wrk in range(num_workers):
-        workerID = wrk + 1
-        local_nodelist, _ = WorkerResources.get_local_nodelist(num_workers, workerID, resources)
-        assert local_nodelist == exp_out[wrk], "local_nodelist returned does not match expected"
 
-    # Spoof current process as each worker and check nodelist.
-    num_workers = 4
-    exp_out = [['knl-0020', 'knl-0021'], ['knl-0022', 'knl-0036'], ['knl-0137', 'knl-0138'], ['knl-0139', 'knl-1234']]
-    for wrk in range(num_workers):
+    # Spoof current process as each worker and check nodelist and other worker resources.
+    for wrk in range(nworkers):
         workerID = wrk + 1
-        local_nodelist, _ = WorkerResources.get_local_nodelist(num_workers, workerID, resources)
-        assert local_nodelist == exp_out[wrk], "local_nodelist returned does not match expected"
+        exp_slots = {exp_out[wrk][0]: [0]}
+        wresources = WorkerResources(nworkers, gresources, workerID)
+        wresources.set_rset_team([wrk])
+        _worker_asserts(wresources, exp_out, exp_slots, wrk, nworkers, 1)
+        del wresources
 
-    # Spoof current process as each worker and check nodelist.
-    num_workers = 1
-    exp_out = [['knl-0020', 'knl-0021', 'knl-0022', 'knl-0036', 'knl-0137', 'knl-0138', 'knl-0139', 'knl-1234']]
-    for wrk in range(num_workers):
+    # 4 Workers ---------------------------------------------------------------
+    nworkers = 4
+    exp_out = [['knl-0020', 'knl-0021'], ['knl-0022', 'knl-0036'],
+               ['knl-0137', 'knl-0138'], ['knl-0139', 'knl-1234']]
+    for wrk in range(nworkers):
         workerID = wrk + 1
-        local_nodelist, _ = WorkerResources.get_local_nodelist(num_workers, workerID, resources)
-        assert local_nodelist == exp_out[wrk], "local_nodelist returned does not match expected"
+        exp_slots = {exp_out[wrk][0]: [0], exp_out[wrk][1]: [0]}
+        wresources = WorkerResources(nworkers, gresources, workerID)
+        wresources.set_rset_team([wrk])
+        _worker_asserts(wresources, exp_out, exp_slots, wrk, nworkers, 2)
+        del wresources
 
-    # Test the best_split algorithm
-    num_workers = 3
-    exp_out = [['knl-0020', 'knl-0021', 'knl-0022'], ['knl-0036', 'knl-0137', 'knl-0138'], ['knl-0139', 'knl-1234']]
-    for wrk in range(num_workers):
+    # 1 Worker ----------------------------------------------------------------
+    nworkers = 1
+    exp_out = [['knl-0020', 'knl-0021', 'knl-0022', 'knl-0036',
+                'knl-0137', 'knl-0138', 'knl-0139', 'knl-1234']]
+
+    # Just write out this one as one worker.
+    exp_slots = {'knl-0020': [0], 'knl-0021': [0], 'knl-0022': [0],
+                 'knl-0036': [0], 'knl-0137': [0], 'knl-0138': [0],
+                 'knl-0139': [0], 'knl-1234': [0]}
+
+    for wrk in range(nworkers):
         workerID = wrk + 1
-        local_nodelist, _ = WorkerResources.get_local_nodelist(num_workers, workerID, resources)
-        assert local_nodelist == exp_out[wrk], "local_nodelist returned does not match expected"
+        # exp_slots - see above
+        wresources = WorkerResources(nworkers, gresources, workerID)
+        wresources.set_rset_team([wrk])
+        _worker_asserts(wresources, exp_out, exp_slots, wrk, nworkers, 8)
+        del wresources
+
+    # 3 Workers (Test the best_split algorithm) -------------------------------
+    nworkers = 3
+    exp_out = [['knl-0020', 'knl-0021', 'knl-0022'],
+               ['knl-0036', 'knl-0137', 'knl-0138'],
+               ['knl-0139', 'knl-1234']]
+    for wrk in range(nworkers):
+        workerID = wrk + 1
+        exp_slots = {exp_out[wrk][0]: [0], exp_out[wrk][1]: [0]}
+        wresources = WorkerResources(nworkers, gresources, workerID)
+        wresources.set_rset_team([wrk])
+
+        # Trying to avoid generic algorithm
+        if len(wresources.local_nodelist) == 3:
+            exp_slots = {exp_out[wrk][0]: [0], exp_out[wrk][1]: [0], exp_out[wrk][2]: [0]}
+            nnodes = 3
+        else:
+            exp_slots = {exp_out[wrk][0]: [0], exp_out[wrk][1]: [0]}
+            nnodes = 2
+        _worker_asserts(wresources, exp_out, exp_slots, wrk, nworkers, nnodes)
+
+    # 16 Workers --------------------------------------------------------------
+    # Multiple workers per node
+    nworkers = 16
+
+    exp_out = [['knl-0020'], ['knl-0020'], ['knl-0021'], ['knl-0021'],
+               ['knl-0022'], ['knl-0022'], ['knl-0036'], ['knl-0036'],
+               ['knl-0137'], ['knl-0137'], ['knl-0138'], ['knl-0138'],
+               ['knl-0139'], ['knl-0139'], ['knl-1234'], ['knl-1234']]
+
+    for wrk in range(nworkers):
+        workerID = wrk + 1
+
+        # If even worker have slot 1
+        if (workerID % 2) == 0:
+            myslot = 1
+        else:
+            myslot = 0
+        exp_slots = {exp_out[wrk][0]: [myslot]}
+        wresources = WorkerResources(nworkers, gresources, workerID)
+        wresources.set_rset_team([wrk])
+        _worker_asserts(wresources, exp_out, exp_slots, wrk, nworkers, 1, 2)
+
+    del os.environ["LIBE_RESOURCES_TEST_NODE_LIST"]
 
 
 # The main tests are same as above - note for when fixtures set up
-def test_get_local_nodelist_central_mode_remove_libE_proc():
+def test_get_local_resources_dedicated_mode_remove_libE_proc():
     mynode = socket.gethostname()
     nodelist_in = ['knl-0020', 'knl-0021', 'knl-0022', 'knl-0036', 'knl-0137', 'knl-0138', 'knl-0139', 'knl-1234']
     with open('node_list', 'w') as f:
@@ -267,67 +351,118 @@ def test_get_local_nodelist_central_mode_remove_libE_proc():
             if i == 3:
                 f.write(mynode + '\n')
 
-    resources = Resources(central_mode=True)
-    resources.add_comm_info(libE_nodes=[mynode])
+    libE_specs = {'dedicated_mode': True}
 
-    # Now mock up some more stuff - so consistent
+    gresources = GlobalResources(libE_specs)
+    gresources.add_comm_info(libE_nodes=[mynode])
 
-    # Spoof current process as each worker and check nodelist.
-    num_workers = 8
+    # 8 Workers ---------------------------------------------------------------
+    nworkers = 8
     exp_out = [['knl-0020'], ['knl-0021'], ['knl-0022'], ['knl-0036'],
                ['knl-0137'], ['knl-0138'], ['knl-0139'], ['knl-1234']]
-    for wrk in range(num_workers):
-        workerID = wrk + 1
-        local_nodelist, _ = WorkerResources.get_local_nodelist(num_workers, workerID, resources)
-        assert local_nodelist == exp_out[wrk], "local_nodelist returned does not match expected"
 
-    # Spoof current process as each worker and check nodelist.
-    num_workers = 4
-    exp_out = [['knl-0020', 'knl-0021'], ['knl-0022', 'knl-0036'], ['knl-0137', 'knl-0138'], ['knl-0139', 'knl-1234']]
-    for wrk in range(num_workers):
+    # Spoof current process as each worker and check nodelist and other worker resources.
+    for wrk in range(nworkers):
         workerID = wrk + 1
-        local_nodelist, _ = WorkerResources.get_local_nodelist(num_workers, workerID, resources)
-        assert local_nodelist == exp_out[wrk], "local_nodelist returned does not match expected"
+        exp_slots = {exp_out[wrk][0]: [0]}
+        wresources = WorkerResources(nworkers, gresources, workerID)
+        wresources.set_rset_team([wrk])
+        _worker_asserts(wresources, exp_out, exp_slots, wrk, nworkers, 1)
+        del wresources
 
-    # Spoof current process as each worker and check nodelist.
-    num_workers = 1
-    exp_out = [['knl-0020', 'knl-0021', 'knl-0022', 'knl-0036', 'knl-0137', 'knl-0138', 'knl-0139', 'knl-1234']]
-    for wrk in range(num_workers):
+    # 4 Workers ---------------------------------------------------------------
+    nworkers = 4
+    exp_out = [['knl-0020', 'knl-0021'], ['knl-0022', 'knl-0036'],
+               ['knl-0137', 'knl-0138'], ['knl-0139', 'knl-1234']]
+    for wrk in range(nworkers):
         workerID = wrk + 1
-        local_nodelist, _ = WorkerResources.get_local_nodelist(num_workers, workerID, resources)
-        assert local_nodelist == exp_out[wrk], "local_nodelist returned does not match expected"
+        exp_slots = {exp_out[wrk][0]: [0], exp_out[wrk][1]: [0]}
+        wresources = WorkerResources(nworkers, gresources, workerID)
+        wresources.set_rset_team([wrk])
+        _worker_asserts(wresources, exp_out, exp_slots, wrk, nworkers, 2)
+        del wresources
 
-    # Test the best_split algorithm
-    num_workers = 3
-    exp_out = [['knl-0020', 'knl-0021', 'knl-0022'], ['knl-0036', 'knl-0137', 'knl-0138'], ['knl-0139', 'knl-1234']]
-    for wrk in range(num_workers):
+    # 1 Worker ----------------------------------------------------------------
+    nworkers = 1
+    exp_out = [['knl-0020', 'knl-0021', 'knl-0022', 'knl-0036',
+                'knl-0137', 'knl-0138', 'knl-0139', 'knl-1234']]
+
+    # Just write out this one as one worker.
+    exp_slots = {'knl-0020': [0], 'knl-0021': [0], 'knl-0022': [0],
+                 'knl-0036': [0], 'knl-0137': [0], 'knl-0138': [0],
+                 'knl-0139': [0], 'knl-1234': [0]}
+
+    for wrk in range(nworkers):
         workerID = wrk + 1
-        local_nodelist, _ = WorkerResources.get_local_nodelist(num_workers, workerID, resources)
-        assert local_nodelist == exp_out[wrk], "local_nodelist returned does not match expected"
+        # exp_slots - see above
+        wresources = WorkerResources(nworkers, gresources, workerID)
+        wresources.set_rset_team([wrk])
+        _worker_asserts(wresources, exp_out, exp_slots, wrk, nworkers, 8)
+        del wresources
+
+    # 3 Workers (Test the best_split algorithm) -------------------------------
+    nworkers = 3
+    exp_out = [['knl-0020', 'knl-0021', 'knl-0022'],
+               ['knl-0036', 'knl-0137', 'knl-0138'],
+               ['knl-0139', 'knl-1234']]
+    for wrk in range(nworkers):
+        workerID = wrk + 1
+        exp_slots = {exp_out[wrk][0]: [0], exp_out[wrk][1]: [0]}
+        wresources = WorkerResources(nworkers, gresources, workerID)
+        wresources.set_rset_team([wrk])
+
+        # Trying to avoid generic algorithm
+        if len(wresources.local_nodelist) == 3:
+            exp_slots = {exp_out[wrk][0]: [0], exp_out[wrk][1]: [0], exp_out[wrk][2]: [0]}
+            nnodes = 3
+        else:
+            exp_slots = {exp_out[wrk][0]: [0], exp_out[wrk][1]: [0]}
+            nnodes = 2
+        _worker_asserts(wresources, exp_out, exp_slots, wrk, nworkers, nnodes)
+        del wresources
+
+    # 16 Workers --------------------------------------------------------------
+    # Multiple workers per node
+    nworkers = 16
+
+    exp_out = [['knl-0020'], ['knl-0020'], ['knl-0021'], ['knl-0021'],
+               ['knl-0022'], ['knl-0022'], ['knl-0036'], ['knl-0036'],
+               ['knl-0137'], ['knl-0137'], ['knl-0138'], ['knl-0138'],
+               ['knl-0139'], ['knl-0139'], ['knl-1234'], ['knl-1234']]
+
+    for wrk in range(nworkers):
+        workerID = wrk + 1
+
+        # If even worker have slot 1
+        if (workerID % 2) == 0:
+            myslot = 1
+        else:
+            myslot = 0
+        exp_slots = {exp_out[wrk][0]: [myslot]}
+        wresources = WorkerResources(nworkers, gresources, workerID)
+        wresources.set_rset_team([wrk])
+        _worker_asserts(wresources, exp_out, exp_slots, wrk, nworkers, 1, 2)
+        del wresources
 
     os.remove('node_list')
 
 
 def test_get_local_nodelist_distrib_mode_host_not_in_list():
     os.environ["LIBE_RESOURCES_TEST_NODE_LIST"] = "knl-[0020-0022,0036,0137-0139,1234]"
-    resources = Resources(nodelist_env_slurm="LIBE_RESOURCES_TEST_NODE_LIST", central_mode=False)
+    resource_info = {'nodelist_env_slurm': "LIBE_RESOURCES_TEST_NODE_LIST"}
+    libE_specs = {'resource_info': resource_info,
+                  'dedicated_mode': False}
 
-    # Spoof current process as each worker and check nodelist.
-    num_workers = 4
+    gresources = GlobalResources(libE_specs)
+    nworkers = 4
     exp_out = ['knl-0022', 'knl-0036']
 
     # Test running distributed mode without current host in list.
     workerID = 2
-    local_nodelist, _ = WorkerResources.get_local_nodelist(num_workers, workerID, resources)
-
-    # Now this should work
-    assert local_nodelist == exp_out, "local_nodelist returned does not match expected"
-    # try:
-    #     local_nodelist, _ = WorkerResources.get_local_nodelist(num_workers, workerID, resources)
-    # except:
-    #     assert 1
-    # else:
-    #     assert 0
+    wresources = WorkerResources(nworkers, gresources, workerID)
+    wresources.set_rset_team([workerID-1])
+    assert wresources.local_nodelist == exp_out, "local_nodelist returned does not match expected"
+    del wresources
 
 
 def test_get_local_nodelist_distrib_mode():
@@ -340,51 +475,58 @@ def test_get_local_nodelist_distrib_mode():
             if i == 3:
                 f.write(mynode + '\n')
 
-    resources = Resources(central_mode=False)
+    libE_specs = {'dedicated_mode': False}
+    gresources = GlobalResources(libE_specs)
+    gresources.add_comm_info(libE_nodes=[mynode])
 
     # Spoof current process as each worker and check nodelist.
-    num_workers = 8
-
-    # Test workerID not in local_nodelist [update: This should now work - check removed]
-    # workerID = 4
-    # try:
-    #     local_nodelist, _ = WorkerResources.get_local_nodelist(num_workers, workerID, resources)
-    # except:
-    #     assert 1
-    # else:
-    #     assert 0
-
+    nworkers = 8
     workerID = 5
     exp_node = mynode  # sname(mynode)
     exp_out = [exp_node]
-    local_nodelist, _ = WorkerResources.get_local_nodelist(num_workers, workerID, resources)
-    assert local_nodelist == exp_out, "local_nodelist returned does not match expected"
+    exp_slots = {exp_node: [0]}
+    wresources = WorkerResources(nworkers, gresources, workerID)
+    wresources.set_rset_team([workerID-1])
+    assert wresources.local_nodelist == exp_out, "local_nodelist returned does not match expected"
 
-    num_workers = 1
+    # Do the rest
+    exp_split = [['knl-0020'], ['knl-0021'], ['knl-0022'], ['knl-0036'],
+                 [exp_node], ['knl-0137'], ['knl-0138'], ['knl-0139']]
+    _worker_asserts(wresources, exp_split, exp_slots, workerID-1, nworkers, 1)
+    del wresources
+
+    nworkers = 1
     workerID = 1
+    wresources = WorkerResources(nworkers, gresources, workerID)
+    wresources.set_rset_team([workerID-1])
     exp_out = ['knl-0020', 'knl-0021', 'knl-0022', 'knl-0036', exp_node, 'knl-0137', 'knl-0138', 'knl-0139']
-    local_nodelist, _ = WorkerResources.get_local_nodelist(num_workers, workerID, resources)
-    assert local_nodelist == exp_out, "local_nodelist returned does not match expected"
+    assert wresources.local_nodelist == exp_out, "local_nodelist returned does not match expected"
+    del wresources
 
-    num_workers = 4
+    nworkers = 4
     workerID = 3
+    wresources = WorkerResources(nworkers, gresources, workerID)
+    wresources.set_rset_team([workerID-1])
     exp_out = [exp_node, 'knl-0137']
-    local_nodelist, _ = WorkerResources.get_local_nodelist(num_workers, workerID, resources)
-    assert local_nodelist == exp_out, "local_nodelist returned does not match expected"
+    assert wresources.local_nodelist == exp_out, "local_nodelist returned does not match expected"
+    del wresources
 
     # Sub-node workers
-    num_workers = 16
-
+    nworkers = 16
     workerID = 9
+    wresources = WorkerResources(nworkers, gresources, workerID)
+    wresources.set_rset_team([workerID-1])
     exp_out = [exp_node]
-    local_nodelist, _ = WorkerResources.get_local_nodelist(num_workers, workerID, resources)
-    assert local_nodelist == exp_out, "local_nodelist returned does not match expected"
+    assert wresources.local_nodelist == exp_out, "local_nodelist returned does not match expected"
+    del wresources
 
     workerID = 10
+    wresources = WorkerResources(nworkers, gresources, workerID)
+    wresources.set_rset_team([workerID-1])
     exp_out = [exp_node]
+    assert wresources.local_nodelist == exp_out, "local_nodelist returned does not match expected"
+    del wresources
 
-    local_nodelist, _ = WorkerResources.get_local_nodelist(num_workers, workerID, resources)
-    assert local_nodelist == exp_out, "local_nodelist returned does not match expected"
     os.remove('node_list')
 
 
@@ -398,103 +540,101 @@ def test_get_local_nodelist_distrib_mode_uneven_split():
             if i == 4:
                 f.write(mynode + '\n')
 
-    resources = Resources(central_mode=False)
-    num_workers = 2
+    libE_specs = {'dedicated_mode': False}
+    gresources = GlobalResources(libE_specs)
+    gresources.add_comm_info(libE_nodes=[mynode])
+    nworkers = 2
 
     # May not be at head of list - should perhaps be warning or enforced
     exp_out_w1 = ['knl-0020', 'knl-0021', 'knl-0022', 'knl-0036', 'knl-0137']
     exp_out_w2 = [exp_node, 'knl-0138', 'knl-0139', 'knl-1234']
+    exp_split = [exp_out_w1, exp_out_w2]
 
     workerID = 2
-    local_nodelist, _ = WorkerResources.get_local_nodelist(num_workers, workerID, resources)
-    assert local_nodelist == exp_out_w2, "local_nodelist returned does not match expected"
+    wresources = WorkerResources(nworkers, gresources, workerID)
+    wresources.set_rset_team([workerID-1])
+    exp_slots = {exp_node: [0], 'knl-0138': [0], 'knl-0139': [0], 'knl-1234': [0]}
+    _worker_asserts(wresources, exp_split, exp_slots, workerID-1, nworkers, 4)
+    del wresources
 
     workerID = 1
-    local_nodelist, _ = WorkerResources.get_local_nodelist(num_workers, workerID, resources)
-    assert local_nodelist == exp_out_w1, "local_nodelist returned does not match expected"
+    wresources = WorkerResources(nworkers, gresources, workerID)
+    wresources.set_rset_team([workerID-1])
+    exp_slots = {'knl-0020': [0], 'knl-0021': [0], 'knl-0022': [0], 'knl-0036': [0], 'knl-0137': [0]}
+    _worker_asserts(wresources, exp_split, exp_slots, workerID-1, nworkers, 5)
+    del wresources
+
     os.remove('node_list')
 
 
-class Fake_comm():
-    def __init__(self, nworkers):
-        self.num_workers = nworkers
-
-    def get_num_workers(self):
-        return self.num_workers
-
-
-def test_worker_resources():
-    os.environ["LIBE_RESOURCES_TEST_NODE_LIST"] = "knl-[0020-0022,0036,0137-0139,1234]"
-    resources = Resources(nodelist_env_slurm="LIBE_RESOURCES_TEST_NODE_LIST", central_mode=True)
-
-    # One worker per node
-    exp_nodelist1 = [['knl-0020'], ['knl-0021'], ['knl-0022'], ['knl-0036'],
-                     ['knl-0137'], ['knl-0138'], ['knl-0139'], ['knl-1234']]
-    num_workers = 8
-    comm = Fake_comm(num_workers)
-    for wrk in range(num_workers):
-        workerID = wrk + 1
-        worker = WorkerResources(workerID, comm, resources)
-        assert worker.num_workers == 8, 'worker.num_workers does not match'
-        assert worker.workerID == workerID, 'worker.workerID does not match'
-        assert worker.local_nodelist == exp_nodelist1[wrk], 'worker.local_nodelist does not match'
-        assert worker.local_node_count == 1, 'worker.local_node_count does not match'
-        assert worker.workers_on_node == 1, 'worker.workers_on_node does not match'
-
-    # Multiple nodes per worker
-    exp_nodelist2 = [['knl-0020', 'knl-0021', 'knl-0022', 'knl-0036'], ['knl-0137', 'knl-0138', 'knl-0139', 'knl-1234']]
-    num_workers = 2
-    comm2 = Fake_comm(num_workers)
-    for wrk in range(num_workers):
-        workerID = wrk + 1
-        worker = WorkerResources(workerID, comm2, resources)
-        assert worker.num_workers == 2, 'worker.num_workers does not match'
-        assert worker.workerID == workerID, 'worker.workerID does not match'
-        assert worker.local_nodelist == exp_nodelist2[wrk], 'worker.local_nodelist does not match'
-        assert worker.local_node_count == 4, 'worker.local_node_count does not match'
-        assert worker.workers_on_node == 1, 'worker.workers_on_node does not match'
-
-    # Multiple workers per node
-    num_workers = 16
-    comm3 = Fake_comm(num_workers)
-    for wrk in range(num_workers):
-        workerID = wrk + 1
-        worker = WorkerResources(workerID, comm3, resources)
-        assert worker.num_workers == 16, 'worker.num_workers does not match'
-        assert worker.workerID == workerID, 'worker.workerID does not match'
-        assert worker.local_nodelist == exp_nodelist1[wrk//2], 'worker.local_nodelist does not match'
-        assert worker.local_node_count == 1, 'worker.local_node_count does not match'
-        assert worker.workers_on_node == 2, 'worker.workers_on_node does not match'
-
-
 def test_map_workerid_to_index():
     num_workers = 4
-    zero_resource_list = []
+    num_rsets = 4
 
+    zero_resource_list = []
+    index_list = ResourceManager.get_index_list(num_workers, num_rsets, zero_resource_list)
     for workerID in range(1, num_workers+1):
-        index = WorkerResources.map_workerid_to_index(num_workers, workerID, zero_resource_list)
+        index = index_list[workerID-1]
         assert index == workerID - 1, "index incorrect. Received: " + str(index)
 
     zero_resource_list = [1]
+    index_list = ResourceManager.get_index_list(num_workers, num_rsets, zero_resource_list)
     for workerID in range(2, num_workers+1):
-        index = WorkerResources.map_workerid_to_index(num_workers, workerID, zero_resource_list)
+        index = index_list[workerID-1]
         assert index == workerID - 2, "index incorrect. Received: " + str(index)
 
     zero_resource_list = [1, 2]
+    index_list = ResourceManager.get_index_list(num_workers, num_rsets, zero_resource_list)
     for workerID in range(3, num_workers+1):
-        index = WorkerResources.map_workerid_to_index(num_workers, workerID, zero_resource_list)
+        index = index_list[workerID-1]
         assert index == workerID - 3, "index incorrect. Received: " + str(index)
 
     zero_resource_list = [1, 3]
+    index_list = ResourceManager.get_index_list(num_workers, num_rsets, zero_resource_list)
+
     workerID = 2
-    index = WorkerResources.map_workerid_to_index(num_workers, workerID, zero_resource_list)
+    index = index_list[workerID-1]
     assert index == 0, "index incorrect. Received: " + str(index)
 
     workerID = 4
-    index = WorkerResources.map_workerid_to_index(num_workers, workerID, zero_resource_list)
+    index = index_list[workerID-1]
     assert index == 1, "index incorrect. Received: " + str(index)
 
 
+def _check_mfile(machinefile, exp_list):
+    with open('machinefile', 'r') as f:
+        i = 0
+        for line in f:
+            index = i//4
+            assert line == exp_list[index]
+            i += 1
+
+
+def test_machinefile_from_resources():
+
+    os.environ["LIBE_RESOURCES_TEST_NODE_LIST"] = "knl-[0020-0022,0036,0137-0139,1234]"
+    resource_info = {'nodelist_env_slurm': "LIBE_RESOURCES_TEST_NODE_LIST"}
+    libE_specs = {'resource_info': resource_info,
+                  'num_resource_sets': 8}
+
+    exp_list = ['knl-0020\n', 'knl-0021\n', 'knl-0022\n', 'knl-0036\n']
+
+    resources = Resources(libE_specs)
+    resources.set_worker_resources(4, 1)
+    resources.worker_resources.set_rset_team([0, 1, 2, 3])
+
+    built_mfile = create_machinefile(resources, num_nodes=4, procs_per_node=4)
+    assert built_mfile, \
+        "machinefile doesn't exist or is empty"
+
+    _check_mfile('machinefile', exp_list)
+
+    # Test replacing older machinefile
+    create_machinefile(resources, machinefile='machinefile', num_nodes=4, procs_per_node=4)
+    _check_mfile('machinefile', exp_list)
+    os.remove('machinefile')
+
+
 if __name__ == "__main__":
     setup_standalone_run()
 
@@ -512,13 +652,14 @@ if __name__ == "__main__":
 
     test_get_global_nodelist_frm_wrklst_file()
     test_remove_libE_nodes()
-    test_get_local_nodelist_central_mode()
-    test_get_local_nodelist_central_mode_remove_libE_proc()
+
+    test_get_local_resources_dedicated_mode()
+    test_get_local_resources_dedicated_mode_remove_libE_proc()
     test_get_local_nodelist_distrib_mode_host_not_in_list()
     test_get_local_nodelist_distrib_mode()
     test_get_local_nodelist_distrib_mode_uneven_split()
 
-    test_worker_resources()
     test_map_workerid_to_index()
+    test_machinefile_from_resources()
 
     teardown_standalone_run()
